;
'use strict';

var _createClass = function() {
	function defineProperties(target, props) {
		for(var i = 0; i < props.length; i++) {
			var descriptor = props[i];
			descriptor.enumerable = descriptor.enumerable || false;
			descriptor.configurable = true;
			if("value" in descriptor) descriptor.writable = true;
			Object.defineProperty(target, descriptor.key, descriptor);
		}
	}
	return function(Constructor, protoProps, staticProps) {
		if(protoProps) defineProperties(Constructor.prototype, protoProps);
		if(staticProps) defineProperties(Constructor, staticProps);
		return Constructor;
	};
}();

function _classCallCheck(instance, Constructor) {
	if(!(instance instanceof Constructor)) {
		throw new TypeError("Cannot call a class as a function");
	}
}

(function() {
	var Animation = function() {
		/**
		 * @param {function} callback wll be run for every update event
		 */
		function Animation(callback) {
			_classCallCheck(this, Animation);

			this.running = false;
			this.callback = callback;
			this.currentTime = 0;
			this.now = 0;
			this.delta = 0;
		}

		_createClass(Animation, [{
			key: 'updateDeltaTime',
			value: function updateDeltaTime() {
				this.now = Date.now();
				this.delta = this.now - this.currentTime;
				this.currentTime = this.now;
			}
		}, {
			key: 'update',
			value: function update() {
				this.updateDeltaTime();
				if(this.running == true) {
					this.callback();
					requestAnimationFrame(this.update.bind(this));
				}
			}
		}, {
			key: 'start',
			value: function start() {
				this.currentTime = Date.now();
				this.running = true;
				this.update();
			}
		}, {
			key: 'stop',
			value: function stop() {
				this.running = false;
			}
		}]);

		return Animation;
	}();

	var Collapser = function() {
		/**
		 * @param {string} html id
		 * @param {boolean} colapse html section
		 * @param {number} duration in ms
		 */
		function Collapser(htmlElementId, collapsed, duration) {
			_classCallCheck(this, Collapser);

			this.id = htmlElementId;
			this.el = document.getElementById(this.id);
			if(this.el == null) {
				console.log('Error element did not found');
				return;
			};
			this.el.style.overflow = 'hidden';
			this.animation = new Animation(this.update.bind(this));
			this.collapsed = false;

			if(collapsed) {
				this.el.style.height = '0px';
				this.collapsed = true;
			}

			this.duration = duration || 200;

			this.height = 0;
			this.maxH = 0;
		}

		_createClass(Collapser, [{
			key: 'getMaxHeight',
			value: function getMaxHeight() {
				var h = this.el.style.height;
				this.el.style.height = null;
				var maxH = this.el.clientHeight;
				this.el.style.height = h;
				return maxH;
			}
		}, {
			key: 'update',
			value: function update() {
				if(this.collapsed == false) {
					this.height -= this.maxH / this.duration * this.animation.delta;
					if(this.height <= 0) {
						this.animation.stop();
						this.collapsed = true;
						this.el.style.height = '0px';
					} else {
						this.el.style.height = this.height + 'px';
					}
				} else {
					this.height += this.maxH / this.duration * this.animation.delta;
					if(this.height >= this.maxH) {
						this.animation.stop();
						this.collapsed = false;
						this.el.style.height = null;
					} else {
						this.el.style.height = this.height + 'px';
					}
				}
			}
		}, {
			key: 'show',
			value: function show() {
				if(!this.collapsed) return;
				this.el.style.height = '0px';
				this.maxH = this.getMaxHeight();
				this.animation.start();
			}
		}, {
			key: 'hide',
			value: function hide() {
				if(this.collapsed) return;
				this.height = this.getMaxHeight();
				this.maxH = this.getMaxHeight();
				this.animation.start();
			}
		}, {
			key: 'toggle',
			value: function toggle() {
				if(this.collapsed) {
					this.show();
				} else {
					this.hide();
				};
			}
		}]);

		return Collapser;
	}();
	//export class

	window['Collapser'] = Collapser;
})();