function defineMethod(type, method, fn) {
	if(method in type.prototype)
		return;
	setUnenum(type.prototype, method, fn);
}

function setUnenum(obj, fld, val) {
	Object.defineProperty(obj, fld, {
		writable: true,
		configurable: true,
		enumerable: false,
		value: val
	});
}

defineMethod(String, 'startsWith', function(str) {
	var str = String(str);
	return this.length >= str.length && this.substring(0, str.length) === str;
});
defineMethod(String, 'endsWith', function(str) {
	var str = String(str);
	return this.length >= str.length && this.substring(this.length - str.length, str.length) === str;
});

/**
 * 立各种flag
 * @param {Object} oldflags
 * @param {Object} newflags
 */
function setFlags(oldflags, newflags) {
	post('setFlags', newflags).then(function() {
		$.extend(oldflags, newflags);
		RSUtils.Toast('设置已保存');
	});
}

function parseDate(d) {
	var m;
	if(typeof d === 'string' && (m = d.match(/^(\d{4})\-(\d\d)\-(\d\d) (\d\d):(\d\d):(\d\d)$/))) {
		d = new Date();
		d.setFullYear(m[1]);
		d.setMonth(m[2] - 1);
		d.setDate(m[3]);
		d.setHours(m[4]);
		d.setMinutes(m[5]);
		d.setSeconds(m[6]);
	} else {
		d = new Date(d);
	}
	return d;
}

function processText(str, raw) {
	raw || (str = escapeHTML(str));
	return str.replace(/(https?:\/\/)?(([\w\-]+\.)+[a-zA-Z]+|(\d+\.){3}\d+)(\/[\w\-\.\/%?#=&+]*)?/g, function(url) {
		url = escapeHTML(url);
		url.match(/^https?:\/\//) || (url = 'http://' + url);
		return '<a class="external" href="' + url + '" style="word-break:break-all">' + url + '</a>';
	});
}

function processFace(str, raw) {
	raw || (str = escapeHTML(str));
	return str.replace(/\{em:(\d+)\}/g, function(m) {
		return '<img src="../../img/face/' + m.substring(4, m.length - 1) + '.gif" />';
	});
}

function escapeHTML(str) {
	var foo = document.createElement('div');
	foo.innerText = str;
	return foo.innerHTML;
}

// 修复 ios 点击输入框外区域，输入框不失去焦点的 bug
mui.os.ios && $(document).on('tap', ':not(input):not(textarea)', function() {
	$('input,textarea').blur();
});

$(document).on('tap', 'a.external', function() {
	var url = this.href;
	url && plus.runtime.openURL(url, function() {
		RSUtils.OpenContentPage({
			url: url
		});
	});
});

/**
 * 格式化日期
 * @param {Object} d 任何可以通过 new Date(d) 转化成合法日期的参数
 */
function formatDate(d) {
	d = parseDate(d);
	var now = new Date();
	if(d.getFullYear() != now.getFullYear()) {
		return d.getFullYear() + '-' + (d.getMonth() + 1) + '-' + d.getDate();
	}
	var pre;
	if(d.getMonth() != now.getMonth() || now.getDate() - d.getDate() > 2) {
		pre = (d.getMonth() + 1) + '-' + d.getDate() + ' ';
	} else if(now.getDate() - d.getDate() == 2) {
		pre = '前天 ';
	} else if(now.getDate() - d.getDate() == 1) {
		pre = '昨天 ';
	} else {
		pre = '';
	}
	return pre + (d.getHours() < 10 ? '0' : '') + d.getHours() + ':' + (d.getMinutes() < 10 ? '0' : '') + d.getMinutes();
}

/**
 * 处理url
 * @param {String} url
 */
function url(url) {
	return url && url.startsWith('/') ? BaseKit.RSHost + '/file' + url : url;
}

/**
 * 从RsUtils.GetUserInfo()获取用户信息
 */
function getUserInfo() {
	var user = RSUtils.GetUserInfo();
	return {
		id: user.id,
		name: user.userName,
		sex: 0,
		head: user.headImgUrl,
		desc: user.signature,
		self: true,
		followCount: 0,
		fanCount: 0,
		likeCount: 0,
		favourCount: 0
	};
}

function getServerTime() {
	var local = +new Date();
	var diff = +getRbItem('timediff') || 0;
	var rtt = +getRbItem('rtt') || 0;
	return local + diff + (rtt / 2);
}

function getRbItem(key) {
	var item = RSUtils.GetItem(RSUtils.DBKeys.RedBook.common);
	if(item && (key in item)) {
		return item[key];
	}
	try {
		return JSON.parse(localStorage[key]);
	} catch(e) {
		// return undefined
	}
}

function setRbItem(key, val) {
	var item = RSUtils.GetItem(RSUtils.DBKeys.RedBook.common);
	if(!item) {
		item = {};
	}
	if(val === null || val === undefined) {
		delete item[key];
	} else {
		item[key] = val;
	}
	RSUtils.SetItem(RSUtils.DBKeys.RedBook.common, item);
	// 操作提示持久化
	if(key.endsWith('-tip')) {
		var flags = {};
		flags['tip_' + key.substring(0, key.length - 4)] = val ? '1' : '0';
		// 存入localStorage，防止清除缓存时丢失
		localStorage[key] = JSON.stringify(val);
		post('setFlags', flags);
	}
}

/**
 * 打开任务（taskDetail）详情页
 * @param {task} o
 */
function goTaskDetail(o) {
	/*RSUtils.OpenContentPage({
		url: '_www/pages/task/task_item.html',
		id: RSUtils.PageId.TaskDetail,
		extras: {
			o: o
		}
	});*/
	RSUtils.OpenContentPage({
		id: RSUtils.PageId.TaskItem,
		url: '_www/pages/task/task_item.html',
		extras: {
			taskId: o.id
		}
	});
}

/**
 * 打开完成任务列表页
 */
function goTask() {
	RSUtils.OpenContentPage({
		url: '_www/pages/task/task.html',
		id: RSUtils.PageId.Task,
		extras: {}
	});
}

/**
 * 打开完成任务列表页
 */
function goTaskFinish() {
	RSUtils.OpenContentPage({
		url: '_www/pages/task/task_finish.html',
		id: RSUtils.PageId.TaskFinish,
		extras: {}
	});
}

/**
 * 打开完成任务列表页
 */
function goTaskOvertime() {
	RSUtils.OpenContentPage({
		url: '_www/pages/task/task_overtime.html',
		id: RSUtils.PageId.TaskOvertime,
		extras: {}
	});
}

/**
 * 打开完成任务列表页
 */
function goTaskAll() {
	RSUtils.OpenContentPage({
		url: '_www/pages/task/task_all.html',
		id: RSUtils.PageId.TaskAll,
		extras: {}
	});
}
// --- AJAX --- //

function get(url, data) {
	return send('GET', url, data || {});
}

function post(url, data) {
	return send('POST', url, data || {});
}

function send(type, api, data) {
	data = $.extend({}, data);
	var url = RSUtils.API.RedBook[api];
	if(!url) {
		return Promise.reject('unknown api ' + api);
	}
	for(var i = 0; i < 10; i++) {
		if(!(i in data)) {
			break;
		}
		url = url.replace('$' + i, data[i]);
		delete data[i];
	}
	return new Promise(function(rs, rj) {
		RSUtils.Load({
			url: url,
			data: data,
			type: type,
			callback: function(r) {
				console.log(r);
				if(r.success) {
					updateOnEvent(r.data);
					rs(r.data);
					rs(r.data1);
				} else {
					rj((r.code || '') + ' ' + (r.message || ''));
				}
			},
			errcallback: function(xhr, err) {
				console.log(err);
				rj(err);
			}
		});
	});
}

function updateOnEvent(o) {
	if(!o) {
		return;
	}
	for(var i in o) {
		(function(type) {
			switch(typeof o[type]) {
				case 'boolean':
					o.id && on(o.id + '.set-bool-' + type, function(v) {
						if(typeof v === 'boolean' && o[type] !== v) {
							o[type] = v;
						}
					});
					break;
				case 'object':
					if(o[type] instanceof Array) {
						for(var i in o[type]) {
							updateOnEvent(o[type][i]);
						}
					} else {
						updateOnEvent(o[type]);
					}
					break;
				default:
					// 其他类型暂不处理
			}
		})(i);
	}
}

function rbready(fn) {
	mui.plusReady(function() {
		new Promise(function(rs) {
			rs(fn());
		}).then(function() {
			window.debug && console.log('rb page inited in  ms');
		}, function(e) {
			console.log(e);
			mui.alert('页面加载时出现异常：' + (e.message || JSON.stringify(e) || '?'), '', function() {
				mui.back();
			});
		});
	});
}

function muiconfirm(text) {
	return new Promise(function(rs, rj) {
		mui.confirm(text, '', ['是', '否'], function(e) {
			e.index == 0 ? rs('用户确认了操作') : rj('用户取消了操作');
		});
	});
}

/**
 * 将页面初始化为下拉刷新、上拉加载
 * @param {Object} cfg
 * 
 * @param {String} cfg.api
 * @param {String} cfg.el = '#wrap'
 * @param {Object} cfg.param = new Object()
 * @param {String} cfg.sort = 'id'
 * @param {Function(Array)} cfg.load
 * @param {Function(Array)} cfg.reload = ()->location.reload()
 */
function initstream(cfg) {
	var api = cfg.api;
	var el = cfg.el || '#wrap';
	var param = cfg.param || {};
	var sort = cfg.sort || 'id';
	mui.init({
		gestureConfig: {
			longtap: !!cfg.longtap
		},
		pullRefresh: {
			container: el,
			up: {
				callback: load
			},
			down: {
				style: 'circle',
				callback: reload
			}
		}
	});
	var from = null;

	function load() {
		get(api, $.extend({
			from: from
		}, param)).then(function(r) {
			cfg.load(r);
		}, function(e) {
			RSUtils.Toast('数据获取失败 ' + e);
		});
	}

	function reload() {
		(cfg.beforeReload || $.noop)()
		get(api, param).then(function(r) {
			cfg.reload(r);
		}, function(e) {
			RSUtils.Toast('数据获取失败 ' + e);
		});
	}
	setTimeout(reload);
}

// --- 事件和消息 --- //

function on(event, handler) {
	RSUtils.addEventListener(event, handler);
}

function broadcast(event, msg) {
	RSUtils.fireEventOnAllWebviews(event, msg);
}

function range(from, to, step) {
	var arr = [];
	for(var i = from; i < to; i += (step || 1)) {
		arr.push(i);
	}
	return arr;
}

var commonMethods = {
	setFlags: setFlags,
	getRbItem: getRbItem,
	setRbItem: setRbItem,
	processText: processText,
	processFace: processFace,
	goTask: goTask,
	goTaskFinish: goTaskFinish,
	goTaskOvertime: goTaskOvertime,
	goTaskAll: goTaskAll,
	goTaskDetail: goTaskDetail,
	escapeHTML: escapeHTML,
	formatDate: formatDate,
	parseDate: parseDate,
	url: url,
	confirm: muiconfirm,
	range: range
};